# jQuiteLight
jQuery Smart Search Query Highlight Plugin

## Features
- Works with Regular Expressions.
- An array of search queries can be applied at once.
- Smart highlight logic is included.
- Additional logic can be applied using `beforeMark` property.
- Each highlight is held separately, so it is easy to remove highlight by index.

## Installation
Installing using node package manager.  
Type the following in your console inside your project directory:  
```
npm install jquitelight
```

Inline HTML including
```html
<!-- Note: jQuery version should not be less than 1.6 -->
<script src="https://ajax.googleapis.com/ajax/libs/jquery/1.6.0/jquery.min.js"></script>
<script src="path/to/plugin/jquitelight.js"></script>
```

## Usage
```js

// basic highlighting
$(".element-to-be-highlighted").mark("text to be highlighted here");

// With RegExp
$(".element-to-be-highlighted").mark(new RegExp("/test.{1,2}/"));

// An array of queries
$(".element-to-be-highlighted").mark(["test", "another-test"]);

// Using options
$(".element-to-be-highlighted").mark(["test", "another-test"], {
	markData: {
		"class": "my-highlight"
	},
	ignoreCase: false
});
// To remove highlight use:
// Get array of highlights
var HIGHLIGHTS = $(".element-to-be-highlighted").data("marker");
// Destroy
HIGHLIGHTS[0].destroy();
```

### Resolving plugin conflicts
```js
// if you have another plugin with such jQuery method
// you can get overridden method by using noConflict method
var prevMarkMethod = $.fn.mark.noConflict();
$.fn.mark.myCoolMark = prevMarkMethod;
```

### Making changes into plugin
```js
// starting from version v1.2.0 plugin core object is linked to jQuery method
var plugin = $.fn.mark.Marker,
	oldWrapStringMethod = plugin.prototype.wrapString;
plugin.prototype.wrapString = function (content) {
	console.log(content);
	return oldWrapStringMethod.call(this, content);
}
```

## Options
`skippedTags`  
Type: array  
Default: `["script", "style"]`  
Expected an array of strings with HTML tag names that should be skipped on highlighting (in lower case).

`ignoreCase`  
Type: bool  
Default: `true`  
Expected a boolean value for ignoring query matching case.

`useSmartBehavior`  
Type: bool  
Default: `false`  
Expected a boolean value for using `RegExp` extending function for string queries.

`beforeMark`  
Type: Function  
Default: `function (text) { return true; }`  
Function deciding match highlight. Accepts a matched string. Should return a boolean value.
 
`afterMark`  
Type: Function  
Default: `function (element) {}`  
Function for manipulating with highlight. Accepts a jQuery element (wrapper of current match).

`markTag`  
Type: String  
Default: `"span"`  
Name of an HTML tag for wrapping matches.

`markData`  
**Notice: DEPRECATED**  
Type: Object  
Default: `{ "class": "marked-text" }`  
Attributes to be applied for wrapper.

`markCss`  
Type: Object  
Default: `{}`  
Css properties to be applied for wrapper.